(* $Id$
 * ----------------------------------------------------------------------
 *
 *)


(* A data endpoint for FTP data connections. This endpoint may be a
 * client or a server (for "passive" and "active" FTP mode, resp.).
 *)

exception Ftp_data_protocol of exn;;

exception Ftp_data_feature_not_implemented;;

type data_operation =
    Data_send_buffer of Netbuffer.t
  | Data_receive_buffer of Netbuffer.t
;;


type data_type =
    Data_ASCII_type
  | Data_EBCDIC_type          (* Not implemented *)
  | Data_image_type
  | Data_local_type of int    (* Not implemented *)
;;


type data_structure =
    Data_file_structure
  | Data_record_structure     (* Not implemented *)
  | Data_page_structure       (* Not implemented *)
;;


type transmission_mode =
    Data_stream_mode
  | Data_block_mode
  | Data_compressed_mode      (* Not implemented *)
;;


type data_options =
    { connection_timeout : float;
      verbose_connection : bool;
      verbose_input : bool;
      verbose_output : bool;
    }
;;
    

class data_session :
  object
    method set_event_system : Unixqueue.event_system -> unit
      (* Sets the event system to use. By default, a private event system
       * is used.
       *)

    method set_server_mode : int
      (* A new server socket is allocated, and the port number of the socket
       * is returned.
       * The socket is not bound to a specific IP address.
       *)

    method set_client_mode : Unix.inet_addr -> int -> unit
      (* Specifies to operate as a client, and to connect to the IP address 
       * and port when the session begins.
       *)

    method set_operation : data_operation -> unit
      (* Set the type of operation, and how to pass data *)

    method set_type : data_type -> unit
      (* Sets the type. Only ASCII and Image types are implemented. *)

    method set_structure : data_structure -> unit
      (* Sets the structure. Only file structure is implemented. *)

    method set_transmission_mode : transmission_mode -> unit
      (* Sets the transmission mode. Only stream and block mode are implemented 
       *)

    method transmit : (bool Lazy.t -> unit) -> unit
      (* transmit f_done:
       * Starts transmission. If the connection is not still open, the
       * necessary steps to establish a connection are performed. For
       * "client mode", the server is contacted. For "server mode", 
       * it is waited until the other side contacts this object. If the
       * connection is still open, it is reused (this is possible for
       * block mode).
       * When the transmission is over, the callback function f_done
       * is invoked with one argument. If "true" is passed, the connection
       * remains open (can be reused for the next transmission). If
       * "false" is passed, the connection is closed. If you apply
       * Lazy.force on the argument, you may also get an exception. This
       * indicates a serious error. 
       * "transmit" attaches to the event system if not already attached.
       *)

    method reset : unit
      (* Closes the connection immediately (if the connection is open).
       * The endpoint is detached from the event system.
       * You may invoke this method to be sure that the connection is
       * down, and that the endpoint is detached.
       * Note that normally only the server is allowed to shut down
       * connections; clients must send the server an "ABORT" command
       * to demand this.
       *)

    method get_options : data_options
      (* Get the configuration options. *)

    method set_options : data_options -> unit
      (* Set the configuration options. *)

  end
;;




(* ======================================================================
 * History:
 * 
 * $Log$
 * 
 *)
